/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package eu.ibacz.contactmgr.backend;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sql.DataSource;

/**
 * This class implements ContactService.
 * 
 * @author Petr Adamek
 */
public class ContactServiceImpl implements ContactService {

    private static final Logger logger = Logger.getLogger(
            ContactServiceImpl.class.getName());
    
    private DataSource ds;

    public void setDataSource(DataSource ds) {
        this.ds = ds;
    }
    
    public void tryCreateTables() {
        checkDataSource();
        try {
            DBUtils.tryCreateTables(ds);
        } catch (SQLException ex) {
            throw new ServiceFailureException("Error when trying to create tables", ex);
        }        
    }
    
    private void checkDataSource() {
        if (ds == null) {
            throw new IllegalStateException("DataSource is not set");
        }
    }
    
    @Override
    public List<Contact> getAll() {
        checkDataSource();
        Connection conn = null;
        PreparedStatement st = null;
        try {
            conn = ds.getConnection();
            st = conn.prepareStatement(
                    "SELECT id, name, phonenumber FROM CONTACTS");
            ResultSet rs = st.executeQuery();
            List<Contact> result = new ArrayList<Contact>();
            while (rs.next()) {
                result.add(rowToContact(rs));
            }
            return result;
        } catch (SQLException ex) {
            String msg = "Error when getting all contacts from DB";
            logger.log(Level.SEVERE, msg, ex);
            throw new ServiceFailureException(msg, ex);
        } finally {
            DBUtils.closeQuietly(conn, st);
        }          
    }

    private Contact rowToContact(ResultSet rs) throws SQLException {
        Contact result = new Contact();
        result.setId(rs.getLong("id"));
        result.setName(rs.getString("name"));
        result.setPhoneNumber(rs.getString("phonenumber"));
        return result;
    }

    @Override
    public void addContact(Contact contact) {
        checkDataSource();
        if (contact == null) {
            throw new IllegalArgumentException("contact is null");
        }
        if (contact.getId() != null) {
            throw new IllegalArgumentException("contact id is already set");
        }
        if (contact.getName() == null || contact.getName().isEmpty()) {
            throw new IllegalArgumentException("contact name is empty");
        }
        if (contact.getPhoneNumber() == null || contact.getPhoneNumber().isEmpty()) {
            throw new IllegalArgumentException("contact phone number is empty");
        }

        Connection conn = null;
        PreparedStatement st = null;
        try {
            conn = ds.getConnection();
            st = conn.prepareStatement(
                    "INSERT INTO CONTACTS (name,phonenumber) VALUES (?,?)",
                    Statement.RETURN_GENERATED_KEYS);
            st.setString(1, contact.getName());
            st.setString(2, contact.getPhoneNumber());

            int count = st.executeUpdate();
            assert count == 1;

            Long id = DBUtils.getId(st.getGeneratedKeys());
            contact.setId(id);

        } catch (SQLException ex) {
            String msg = "Error when inserting contact into db";
            logger.log(Level.SEVERE, msg, ex);
            throw new ServiceFailureException(msg, ex);
        } finally {
            DBUtils.closeQuietly(conn, st);
        }
    }

    @Override
    public Contact getContact(Long id) {

        checkDataSource();
        
        if (id == null) {
            throw new IllegalArgumentException("id is null");
        }
        
        Connection conn = null;
        PreparedStatement st = null;
        try {
            conn = ds.getConnection();
            st = conn.prepareStatement(
                    "SELECT id, name, phonenumber FROM CONTACTS WHERE id = ?");
            st.setLong(1, id);
            ResultSet rs = st.executeQuery();
            if (rs.next()) {
                Contact result = rowToContact(rs);                
                if (rs.next()) {
                    throw new ServiceFailureException(
                            "Internal integrity error: more contacts with the same id found!");
                }
                return result;
            } else {
                return null;
            }
        } catch (SQLException ex) {
            String msg = "Error when getting contact with id = " + id + " from DB";
            logger.log(Level.SEVERE, msg, ex);
            throw new ServiceFailureException(msg, ex);
        } finally {
            DBUtils.closeQuietly(conn, st);
        }
    }
}
